/************************************************************************
 *   IRC - Internet Relay Chat, contrib/m_map.c
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_map.c,v 1.13 2002/04/13 14:26:43 reddawg Exp $
 */

#include <assert.h>
#include "ircd.h"
#include "config.h"
#include "client.h"
#include "numeric.h"
#include "send.h"
#include "s_conf.h"

#define USER_COL       50 /* display | Users: %d at col 50 */

static char buf[BUFSIZE];

static void dump_map(struct Client *cptr,struct Client *root_p, char *pbuf);

/* m_map
**	parv[0] = sender prefix
*/

int m_map(struct Client *cptr, struct Client *sptr, int parc, char *parv[]) {
  dump_map(cptr,&me,buf);
  sendto_one(cptr, form_str(RPL_MAPEND), me.name, cptr->name);
  return;
  return;
  }

/*
** mo_map
**      parv[0] = sender prefix
*/
static void mo_map(struct Client *cptr, struct Client *sptr,
                    int parc, char *parv[])
{
  dump_map(cptr,&me,buf);
  sendto_one(cptr, form_str(RPL_MAPEND), me.name, cptr->name);
}

/*
** dump_map
**   dumps server map, called recursively.
*/
static void dump_map(struct Client *cptr,struct Client *root_p, char *pbuf)
{
  int cnt = 0, i = 0, len;
  int users = 0;
  struct Client *server_p,*user_p;
        
  *pbuf= '\0';
       
  strncat(pbuf,root_p->name,BUFSIZE - ((size_t) pbuf - (size_t) buf));
  len = strlen(buf);
  buf[len] = ' ';
	
  if (len < USER_COL)
  {
     for (i = len+1; i < USER_COL; i++)
     {
       buf[i] = ' ';
     }
  }
	
  /* FIXME: add serv->usercnt */
  for( user_p = root_p->serv->users; user_p; user_p = user_p->lnext )
    users++;
  snprintf(buf + USER_COL, BUFSIZE - USER_COL,
           " Users: %5d (%4.1f%%)", users,
           100 * (float) users / (float) Count.total);
        
  sendto_one(cptr, form_str(RPL_MAP),me.name,cptr->name,buf);
        
  if ((server_p = root_p->serv->servers))
  {
    for (; server_p; server_p = server_p->lnext)
    {
      cnt++;
    }
    
    if (cnt)
    {
      if (pbuf > buf + 3)
      {
        pbuf[-2] = ' ';
        if (pbuf[-3] == '`')
          pbuf[-3] = ' ';
      }
    }
  }
  for (i = 1,server_p = root_p->serv->servers; server_p; server_p=server_p->lnext)
  {
    *pbuf = ' ';
    if (i < cnt)
      *(pbuf + 1) = '|';
    else
      *(pbuf + 1) = '`';
      
    *(pbuf + 2) = '-';
    *(pbuf + 3) = ' ';
    dump_map(cptr,server_p,pbuf+4);
 
    i++;
   }
}

